@@ -1,4 +1,4 @@
-// Copyright (C) 2015 Free Software Foundation, Inc.
+// Copyright (C) 2015-2024 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -17,8 +17,7 @@
 
 // Jonathan Wakely  <jwakely@redhat.com>
 
-// { dg-options "-std=gnu++11" }
-// { dg-do compile }
+// { dg-do compile { target c++11 } }
 
 #include <unordered_map>
 
@@ -28,3 +27,7 @@ using alloc_type = test_type::allocator_type;
 
 test_type h1(10, alloc_type());
 test_type h2(10, hasher_type(), alloc_type());
+test_type h3(h1.begin(), h1.end(), 10, alloc_type());
+test_type h4(h1.begin(), h1.end(), 10, hasher_type(), alloc_type());
+test_type h5({ { 1, 1 } }, 10, alloc_type());
+test_type h6({ { 1, 1 } }, 10, hasher_type(), alloc_type());
