@@ -1,6 +1,6 @@
-// { dg-options "-std=gnu++11" }
+// { dg-do run { target c++11 } }
 
-// Copyright (C) 2008-2016 Free Software Foundation, Inc.
+// Copyright (C) 2008-2024 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -26,7 +26,6 @@
 
 void test01()
 {
-  bool test __attribute__((unused)) = true;
   std::vector<int> c(3);    
   std::vector<int>::iterator i = c.end(), j;
 
@@ -37,7 +36,6 @@ void test01()
 
 void test02()
 {
-  bool test __attribute__((unused)) = true;
   std::list<int> c(3);    
   std::list<int>::iterator i = c.end(), j;
 
