@@ -1,6 +1,6 @@
-// 2003-05-01  Petur Runolfsson  <peturr02@ru.is>
+// 2003-05-01 Petur Runolfsson  <peturr02@ru.is>
 
-// Copyright (C) 2000-2020 Free Software Foundation, Inc.
+// Copyright (C) 2000-2024 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -19,15 +19,20 @@
 
 #include <iostream>
 
-void test09()
+// libstdc++/2523
+void test03()
 {
-  std::wcout << L"Enter favorite beach: ";
-  std::wcin.ignore(2048, '\n');
+  using namespace std;
+  ios_base::sync_with_stdio(false);
+
+  int i;
+  wcin >> i;
+  wcout << "i == " << i << endl;
 }
 
 int 
 main()
 {
-  test09();
+  test03();
   return 0;
 }
